#############################################################
# Authors:                                                  #
# Andreas Gericke (IGB Berlin, gericke@igb-berlin.de) and   #
# Martin Tschikof (BOKU Vienna, martin.tschikof@boku.ac.at) #
# for: prioritisation of floodplain (segments)              #
# based on modelled nutrient emissions and loads as well as #
# their retention in floodplains and streams                #
# DTP Interreg IDES, Oct 2021                               #
#############################################################

LANGUAGE='en'
#install.packages("tidyverse")
library(tidyverse)

# Please enter your working directory
wd <- "D:/..."
if (!dir.exists(wd)) stop("Please change the wd variable to an existing folder name")
setwd(wd)

# Load data===========================================================================================
# read file with annual output of MONERIS model (model discharge from adjusted E-HYPE data)
# MONERIS setup of 3rd update of DRBMP of the ICPDR
# the file contains discharge and loads for analytical units which have been assigned to floodplains
# and the values of their upstream neighbours (*_up)
dhy = read.delim("export_fp_Sep_2021_E-HYPE_LT_adjusted_buffer_monthly_merged_fixed.txt_monthly_merged_loads_no_remob_2021.txt",
                 header = T, quote = "", sep = "\t", dec = ",")%>%
# the file contains two TP results:
# a) based on recalibrated in-stream retention without remobilisation (Sep 2021) and
# b) with remobilisation (Oct 2021) (*excel* columns)
# the latter approach gives higher loads, the overall agreement to observed load is slightly better
# we drop a) and rename the excel columns to keep the rest of the script unchanged
# so if version a) is needed, simply comment the select() and rename()
# the loads are almost perfectly correlated, so the choice of the approach should not affect the prioritisation
  select(-LoadTP_tons,-LoadTP_tons_up,-ConcTP_mgL_up,-ConcTP_mgL,
         -RetentionInMainRiversDIN_THL,-RetentionInMainRiversTN_THL,-Ret_TP_mainRiver_excel,
         -Ret_TN_mainRiver_monthly,-Ret_TP_mainRiver_monthy,-Ret_NO3_mainRivers_monthly)%>% #remove fomer TP und retention values
  rename(LoadTP_tons = LoadTP_excel_tons,
         LoadTP_tons_up = LoadTP_excel_tons_up,
         ConcTP_mgL = ConcTP_excel_mgL,
         ConcTP_mgL_up = ConcTP_mgL_excel_up)
# input file with intersected AU and FP (floodplains)
# floodplains can intersect with several analytical units (AU) of MONERIS
# we manually assigned the FPs to AUs following their main flow direction
# for each floodplain have its area share in each assigned AU which allows us
# to calculated area weighted means of Moneris results later on
# the file also contains the location of FPs within AUs (e.g. up, down)
# which allows us to assign the AU or the upstream values (or some average)
dau = read.delim("AU_intersect_FP_DRSV_edit.txt", header = T, quote = "", sep = "\t")
# input file with intersected AU and SEG (10 km segments)
dau_seg = read.delim("AU_intersect_Seg.txt", header = T, quote = "", sep = "\t") 
# calculated (modelled) values for nutrient retention in floodplains. Averaged for FP and SEG:
#a) denitrification potential, adapted from Heinen (2006)  
#b) TP deposition  potential, from Schulz-Zunkel et al. (2021)
#c) flooding frequencies, adapted from Schleuter (2016)
meanret = read.csv("Mean_retention_flooding_days.csv", sep=",", dec=".")
meanret_seg = read.csv("Mean_retention_flooding_days_seg.csv", sep=",", dec=".")
# in addition WA (water surface area) and WT (water temperature) for in-stream retention
# Moneris also calculates retention but we had more detailed data for the floodplains,
# so the in-stream retention is re-calculated below using the (T)HL approach of Moneris
# we follow the new Moneris approach *without remobilisation* because
# a) the remobilisation is based on monthly Q changes but for IDES we are interested in average conditions
# b) the calculated loads are extremely well correlated, so is the instream retention
# as the prioritisation is not based on absolute values but percentiles, it will not be affected
wa_fp = read.csv("River_area_aFP.csv", sep=";", dec=",")%>%
  filter(!grepl('BG_YN_AFP_06|SI_KR_AFP_', DFGIS_ID))
wa_seg = read.csv("River_area_segments_aFP.csv", sep=";", dec=",")%>%
  filter(!grepl('BG_YN_AFP_06|SI_KR_AFP_', DFGIS_ID))
wt_au = read.csv("AU_Wassertemperatur.csv", sep=";", dec=",")

# ==== group and join
dau_hy <- dau%>%
  group_by(AU_revised,DFGIS_ID, Location.AU.revised,Transbound)%>%
  summarise(area_union_m2=sum(area_union_m2), Share.FP.in.AU=sum(Share.FP.in.AU), AU_Area_km2=mean(AU_Area1_km2))%>%
  inner_join(., dhy, by = c("AU_revised"="AU"))%>%
  inner_join(., wa_fp, by = "DFGIS_ID")%>%
  inner_join(., select(wt_au, AU, Annual_temp), by = c("AU_revised"="AU"))



# Connect variables (columns) for AUs to FPs and SEGs =========================================================================

# Calculate sum of weighted emissions per FP
# only use relevant pathways for floodplains: ER = Erosion, SR = Surface runoff (possible extension: GW = groundwater or subsurface flow)
# we divided the total emissions (in tons) by the AU area to avoid artefacts of large AUs
# the AU areas differ widely, large AUs will likely have high total emissions but the floodplains might only cover a small part of these AU
d_em = dau_hy %>% 
  mutate(Emission_TN = Emission_TN * (Emission_TN_SR_share + Emission_TN_ER_share) / AU_Area_km2,
         Emission_TP = Emission_TP * (Emission_TP_SR_share + Emission_TP_ER_share) / AU_Area_km2,
         Emission_NO3 = Emission_NO3 * (Emission_TN_SR_share + Emission_TN_ER_share) / AU_Area_km2) %>%
  mutate(Emission_TN=Share.FP.in.AU * Emission_TN,
         Emission_TP=Share.FP.in.AU * Emission_TP,
         Emission_NO3=Share.FP.in.AU * Emission_NO3,
         # average AU slope, needed for calculation of TP retention below
         Slope100Mean=Share.FP.in.AU * Slope100Mean) %>%
  group_by(DFGIS_ID)%>% 
  summarise(.,across(c(Emission_TN,Emission_TP,Emission_NO3,Slope100Mean),sum), 
            across(c(Area_river_m2, Annual_temp),mean))

# for loads and discharge, we consider the FP location in AU
# e.g. downstream means near the outlet of the assigned AU
d <- dau_hy %>%
  mutate(
    LoadTN_tons = case_when(
      Location.AU.revised=="down" ~ LoadTN_tons,
      Location.AU.revised=="up" ~ LoadTN_tons_up,
      Location.AU.revised=="mid" ~ (LoadTN_tons+LoadTN_tons_up)/2,
      Location.AU.revised=="" ~ (LoadTN_tons+LoadTN_tons_up)/2,
      Location.AU.revised=="up-mid" ~ (LoadTN_tons+2*LoadTN_tons_up)/3,
      Location.AU.revised=="mid-down" ~ (2*LoadTN_tons+LoadTN_tons_up)/3),
    
    LoadNO3_tons = case_when(
      Location.AU.revised=="down" ~ LoadNO3_tons,
      Location.AU.revised=="up" ~ LoadNO3_tons_up,
      Location.AU.revised=="mid" ~ (LoadNO3_tons+LoadNO3_tons_up)/2,
      Location.AU.revised=="" ~ (LoadNO3_tons+LoadNO3_tons_up)/2,
      Location.AU.revised=="up-mid" ~ (LoadNO3_tons+2*LoadNO3_tons_up)/3,
      Location.AU.revised=="mid-down" ~ (2*LoadNO3_tons+LoadNO3_tons_up)/3),  
    
    LoadTP_tons = case_when(
      Location.AU.revised=="down" ~ LoadTP_tons,
      Location.AU.revised=="up" ~ LoadTP_tons_up,
      Location.AU.revised=="mid" ~ (LoadTP_tons+LoadTP_tons_up)/2,
      Location.AU.revised=="" ~ (LoadTP_tons+LoadTP_tons_up)/2,
      Location.AU.revised=="up-mid" ~ (LoadTP_tons+2*LoadTP_tons_up)/3,
      Location.AU.revised=="mid-down" ~ (2*LoadTP_tons+LoadTP_tons_up)/3),
    
    Discharge_m3s = case_when(
      Location.AU.revised=="down" ~ Discharge_m3s,
      Location.AU.revised=="up" ~ Discharge_m3s_up,
      Location.AU.revised=="mid" ~ (Discharge_m3s+Discharge_m3s_up)/2,
      Location.AU.revised=="" ~ (Discharge_m3s+Discharge_m3s_up)/2,
      Location.AU.revised=="up-mid" ~ (Discharge_m3s+2*Discharge_m3s_up)/3,
      Location.AU.revised=="mid-down" ~ (2*Discharge_m3s+Discharge_m3s_up)/3),
    
    Discharge_m3 = case_when(
      Location.AU.revised=="down" ~ Discharge_m3,
      Location.AU.revised=="up" ~ Discharge_m3_up,
      Location.AU.revised=="mid" ~ (Discharge_m3+Discharge_m3_up)/2,
      Location.AU.revised=="" ~ (Discharge_m3+Discharge_m3_up)/2,
      Location.AU.revised=="up-mid" ~ (Discharge_m3+2*Discharge_m3_up)/3,
      Location.AU.revised=="mid-down" ~ (2*Discharge_m3+Discharge_m3_up)/3)
  ) %>%
  group_by(DFGIS_ID, AU_revised)%>%
  summarise_at(vars(LoadTN_tons,LoadNO3_tons, LoadTP_tons, Discharge_m3s,
                    Discharge_m3, Share.FP.in.AU), sum)

# Take area weighted mean of Load and Q for FP and SEG based on area share of the different AU
# FP:
d_mean <- d %>%
  mutate(
       LoadTN_tons=Share.FP.in.AU*LoadTN_tons,
       LoadNO3_tons=Share.FP.in.AU*LoadNO3_tons,
       LoadTP_tons=Share.FP.in.AU*LoadTP_tons,
       Discharge_m3=Share.FP.in.AU*Discharge_m3,
       Discharge_m3s=Share.FP.in.AU*Discharge_m3s) %>%
  group_by(DFGIS_ID) %>%
  summarize(
            LoadTN_tons=sum(LoadTN_tons),
            LoadNO3_tons=sum(LoadNO3_tons),
            LoadTP_tons=sum(LoadTP_tons),
            Discharge_m3=sum(Discharge_m3),
            Discharge_m3s=sum(Discharge_m3s))%>% 
  # calculate concentration from load and discharge
  mutate(ConcTN_mgL = LoadTN_tons * 1e6 / Discharge_m3,
         ConcNO3_mgL = LoadNO3_tons * 1e6 / Discharge_m3,
         ConcTP_mgL = LoadTP_tons * 1e6 / Discharge_m3)%>%
  # join with FP retention
  inner_join(., meanret, by = c("DFGIS_ID")) %>%
  # join with emission
  inner_join(., d_em, by = c("DFGIS_ID"))

# SEG: inherit the AU-based values from FPs
# note: Share.FP.in.AU refers to segments not floodplains
d_seg <- d_mean %>% 
   select(c(DFGIS_ID, matches("Discharge|T[NP]|NO3|temp"))) %>% select(-TPret_mean) %>%
  inner_join(., dau_seg %>% group_by(IDESGIS_ID, DFGIS_ID, Share.FP.in.AU) %>% summarise() %>% ungroup(),
             by = c("DFGIS_ID")) %>%
 # area-weighted mean
  mutate(
    Emission_TN=Share.FP.in.AU * Emission_TN, 
    Emission_NO3=Share.FP.in.AU * Emission_NO3,
    Emission_TP=Share.FP.in.AU * Emission_TP,
    LoadTN_tons=Share.FP.in.AU*LoadTN_tons,
    LoadNO3_tons=Share.FP.in.AU*LoadNO3_tons,
    LoadTP_tons=Share.FP.in.AU*LoadTP_tons,
    Discharge_m3=Share.FP.in.AU*Discharge_m3,
    Discharge_m3s=Share.FP.in.AU*Discharge_m3s) %>%
  group_by(IDESGIS_ID) %>%
  summarize(
    DFGIS_ID = paste(unique(DFGIS_ID), collapse=", "),
    Emission_TN=sum(Emission_TN),
    Emission_NO3=sum(Emission_NO3),
    Emission_TP=sum(Emission_TP),
    LoadTN_tons=sum(LoadTN_tons),
    LoadNO3_tons=sum(LoadNO3_tons),
    LoadTP_tons=sum(LoadTP_tons),
    Discharge_m3=sum(Discharge_m3),
    Discharge_m3s=sum(Discharge_m3s),
    Annual_temp = mean(Annual_temp))%>% 
  # calculate concentration from load and discharge
  mutate(ConcTN_mgL = LoadTN_tons * 1e6 / Discharge_m3,
         ConcNO3_mgL = LoadNO3_tons * 1e6 / Discharge_m3,
         ConcTP_mgL = LoadTP_tons * 1e6 / Discharge_m3)%>%
  # join with SEG retention
  inner_join(., meanret_seg, by = c("IDESGIS_ID")) 


# Caclulation of in-stream retention in FP and SEG ==========================================================
# following the Moneris approach
# the instream retention is based on the hydraulic load = HL (in m/yr) (cf. Venohr et al. 2011)
# which can be calculated as Q (in m3/yr) / water surface area (in m2)

# Used constants
# original Moneris TP approach (HL, without remobilization)
CR3 = 25.74
# newly calibrated TP approach (HL, without remobilization)
# the new approach 
CR3.1 = 56.3
CR4 = -1
# Moneris approach for DIN (THL)
CR5 = 8.58 
CR6 = 0.067 
CR7 = -1

# FP:
d_fp_ret = d_mean %>%
  # original Moneris TP approach
  #mutate(RetentionInMainRiversTP_mt = 1-(1 / (1 + CR3 * (Discharge_m3/Area_river_m2) ^ CR4)))%>%
  # newly calibrated TP approach
  mutate(RetentionInMainRiversTP_mt = 1-(1 / (1. + 56.3 / Slope100Mean * (Discharge_m3/Area_river_m2) ^ CR4))) %>%
  mutate(RetentionInMainRiversDIN_mt = 1-(1 / (1 + CR5 * exp(CR6 * Annual_temp) * (Discharge_m3/Area_river_m2) ^ CR7)))

# SEG:
wa_seg = wa_seg %>%
  filter(.,Type=="River")%>%
  group_by(IDESGIS_ID)%>%
  summarise(Area_river_seg_m2 =sum(Area_river_seg_m2))%>%
  # Create New_ID to join SEG with WA
  mutate(New_ID=paste(str_sub(IDESGIS_ID,1,-13),str_sub(IDESGIS_ID,-3,-1)))%>% 
  select(-IDESGIS_ID)

d_seg_ret = d_seg %>% 
  mutate(New_ID=paste(str_sub(IDESGIS_ID,1,-11),str_sub(IDESGIS_ID,-3,-1)))%>%
  inner_join(., wa_seg, by = "New_ID")%>%
  mutate(RetentionInMainRiversTP_mt = 1-(1 / (1 + CR3 * (Discharge_m3/Area_river_seg_m2) ^ CR4)))%>%
  mutate(RetentionInMainRiversDIN_mt = 1-(1 / (1 + CR5 * exp(CR6 * Annual_temp) * (Discharge_m3/Area_river_seg_m2) ^ CR7)))

# Calculate relevance of FP and SEG for water quality: basin wide and per country ===============================================
# the absolute values are transformed to percentiles as class values (1..n), her
# the higher the (average) class value the higher the prioritization

# number of percentiles (classes)
n_classes <- 3  # = terciles

# optional weighting factors:
wem = 1 #weighting factor emissions
wco = 1 #weighting factor concentrations
wfr = 1 #weighting factor floodplain retention
wir = 1 #weighting factor in-stream retention
wfd = 1 #weighting factor flooding days
# allow only positive weighting factors
# by doing so, we avoid that all factors for any weighted mean are zero, which would result in NaN
stopifnot(min(c(wem, wco, wfr, wir, wfd)) > 0)

# FP:
# basin-wide prioritization
fp_rel_bw=d_fp_ret%>%  
  mutate(ConcNO3_mgL_rank = ntile(ConcNO3_mgL, n_classes))%>%
  mutate(ConcTP_mgL_rank = ntile(ConcTP_mgL, n_classes))%>%
  mutate(Denit_Conn_mean_rank = ntile(Denit_Conn_mean, n_classes))%>%
  mutate(Denit_WaW_mean_rank = ntile(Denit_WaW_mean, n_classes))%>%
  mutate(TPret_mean_rank = ntile(TPret_mean, n_classes))%>%
  mutate(Flooding_days_mean_rank = ntile(Flooding_days_mean, n_classes))%>%
  mutate(Emission_NO3_rank = ntile(Emission_NO3, n_classes))%>%
  mutate(Emission_TP_rank = ntile(Emission_TP, n_classes))%>%
  mutate(TPret_river_rank = ntile(RetentionInMainRiversTP_mt, n_classes))%>%
  mutate(DINret_river_rank = ntile(RetentionInMainRiversDIN_mt, n_classes)) %>%
  mutate(scale="basin")

# nation-wide prioritization
fp_rel_nw=d_fp_ret%>%    
  # FPs along national borders can be assigned to multiple countries,
  # so, we split transboundary FP into separate lines 
  separate_rows(Cntr_Name, sep=", *") %>%
  group_by(Cntr_Name)%>%
  mutate(ConcNO3_mgL_rank = ntile(ConcNO3_mgL, n_classes))%>%
  mutate(ConcTP_mgL_rank = ntile(ConcTP_mgL, n_classes))%>%
  mutate(Denit_Conn_mean_rank = ntile(Denit_Conn_mean, n_classes))%>%
  mutate(Denit_WaW_mean_rank = ntile(Denit_WaW_mean, n_classes))%>%
  mutate(TPret_mean_rank = ntile(TPret_mean, n_classes))%>%
  mutate(Flooding_days_mean_rank = ntile(Flooding_days_mean, n_classes))%>%
  mutate(Emission_NO3_rank = ntile(Emission_NO3, n_classes))%>%
  mutate(Emission_TP_rank = ntile(Emission_TP, n_classes))%>%
  mutate(TPret_river_rank = ntile(RetentionInMainRiversTP_mt, n_classes))%>%
  mutate(DINret_river_rank = ntile(RetentionInMainRiversDIN_mt, n_classes))%>%
  mutate(scale="national")%>%
  # Increase rank in countries with <3 floodplains by (n_classes - max(rank))
  # to make sure that the highest rank is always n_classes
  mutate(across(ends_with("_rank"), ~.x + (n_classes - max(.x))))


# join scales and evaluate relevance as average class value
# take mean percntiles for relevant nutrient (N/P) and pathway (from emission/ from river inputs)
fp_rel <- bind_rows(fp_rel_bw, fp_rel_nw) %>%
  rowwise() %>% 
  mutate(NO3_emission_rel= weighted.mean(c(Emission_NO3_rank,Denit_WaW_mean_rank), w=c(wem, wfr)))%>% 
  mutate(NO3_conc_rel= weighted.mean(c(ConcNO3_mgL_rank, Denit_WaW_mean_rank, Flooding_days_mean_rank, DINret_river_rank), w=c(wco, wfr, wfd, wir)))%>%
  mutate(P_emission_rel= weighted.mean(c(Emission_TP_rank,TPret_mean_rank), w=c(wem, wfr)))%>%
  mutate(P_conc_rel= weighted.mean(c(ConcTP_mgL_rank, TPret_mean_rank, Flooding_days_mean_rank, TPret_river_rank), w=c(wco, wfr, wfd, wir)))


# SEG:
# basin-wide prioritization
seg_rel_bw=d_seg_ret%>%
  mutate(ConcNO3_mgL_rank = ntile(ConcNO3_mgL, n_classes))%>%
  mutate(ConcTP_mgL_rank = ntile(ConcTP_mgL, n_classes))%>%
  mutate(Denit_Conn_mean_rank = ntile(Denit_Conn_mean, n_classes))%>%
  mutate(Denit_WaW_mean_rank = ntile(Denit_WaW_mean, n_classes))%>%
  mutate(TPret_mean_rank = ntile(TPret_mean, n_classes))%>%
  mutate(Flooding_days_mean_rank = ntile(Flooding_days_mean, n_classes))%>%
  mutate(Emission_NO3_rank = ntile(Emission_NO3, n_classes))%>%
  mutate(Emission_TP_rank = ntile(Emission_TP, n_classes))%>%
  mutate(TPret_river_rank = ntile(RetentionInMainRiversTP_mt, n_classes))%>%
  mutate(DINret_river_rank = ntile(RetentionInMainRiversDIN_mt, n_classes))%>%  
  mutate(scale="basin")

# nation-wide prioritization
seg_rel_nw=d_seg_ret%>%
  separate_rows(Cntr_Name, sep="_") %>%
  # standardize country codes with codes used for FP
  mutate(Cntr_Name=ifelse(Cntr_Name=="BGR", "BG", ifelse(Cntr_Name=="GER", "DE", Cntr_Name))) %>% 
  group_by(Cntr_Name)%>%
  # separate_rows(Cntr_Name, sep=", *") %>%
  # group_by(Cntr_Name)%>%
  mutate(ConcNO3_mgL_rank = ntile(ConcNO3_mgL, n_classes))%>%
  mutate(ConcTP_mgL_rank = ntile(ConcTP_mgL, n_classes))%>%
  mutate(Denit_Conn_mean_rank = ntile(Denit_Conn_mean, n_classes))%>%
  mutate(Denit_WaW_mean_rank = ntile(Denit_WaW_mean, n_classes))%>%
  mutate(TPret_mean_rank = ntile(TPret_mean, n_classes))%>%
  mutate(Flooding_days_mean_rank = ntile(Flooding_days_mean, n_classes))%>%
  mutate(Emission_NO3_rank = ntile(Emission_NO3, n_classes))%>%
  mutate(Emission_TP_rank = ntile(Emission_TP, n_classes))%>%
  mutate(TPret_river_rank = ntile(RetentionInMainRiversTP_mt, n_classes))%>%
  mutate(DINret_river_rank = ntile(RetentionInMainRiversDIN_mt, n_classes))%>%
  mutate(scale="national")%>%
  # Increase rank in countries with <3 floodplains by (n_classes - max(rank))
  mutate(across(ends_with("_rank"), ~.x + (n_classes - max(.x))))

  # evaluate relevance
seg_rel <- bind_rows(seg_rel_bw, seg_rel_nw) %>%
  rowwise() %>% 
  mutate(NO3_emission_rel= weighted.mean(c(Emission_NO3_rank, Denit_WaW_mean_rank), w=c(wem, wfr)))%>%
  mutate(NO3_conc_rel= weighted.mean(c(ConcNO3_mgL_rank, Denit_WaW_mean_rank, Flooding_days_mean_rank, DINret_river_rank), w=c(wco, wfr, wfd, wir)))%>%
  mutate(P_emission_rel= weighted.mean(c(Emission_TP_rank, TPret_mean_rank), w=c(wem, wfr)))%>%
  mutate(P_conc_rel= weighted.mean(c(ConcTP_mgL_rank, TPret_mean_rank, Flooding_days_mean_rank, TPret_river_rank), w=c(wco, wfr, wfd, wir)))


# export evaluation tables (dec=".", sep=",") =======================
fp_out_file <- "Floodplain_WQ_relevance.csv"
seg_out_file <- "Segment10km_WQ_relevance.csv"
if (file.exists(fp_out_file)) warning(paste0("Overwrite existing '",fp_out_file,"'"))
if (file.exists(fp_out_file)) warning(paste0("Overwrite existing '",seg_out_file,"'"))
write_excel_csv(fp_rel, file=fp_out_file)
write_excel_csv(seg_rel, file=seg_out_file)
print("Ranking files successfully written")
